@@ -30,6 +30,7 @@ from sqlalchemy.ext.declarative.base import _get_immediate_cls_attr
 from sqlalchemy.inspection import inspect
 from sqlalchemy.orm.base import class_mapper
 from sqlalchemy.orm.mapper import Mapper
+from sqlalchemy.orm.relationships import RelationshipProperty
 from sqlalchemy.orm.session import Session
 from sqlalchemy.sql.schema import Column, MetaData
 from sqlalchemy.sql.type_api import TypeEngine
@@ -43,7 +44,6 @@ from cardinal_pythonlib.logs import BraceStyleAdapter
 
 if TYPE_CHECKING:
     from sqlalchemy.orm.state import InstanceState
-    from sqlalchemy.orm.relationships import RelationshipProperty
     from sqlalchemy.sql.schema import Table
 
 log = logging.getLogger(__name__)
@@ -121,30 +121,58 @@ class SqlAlchemyAttrDictMixin(object):
 # Traverse ORM relationships (SQLAlchemy ORM)
 # =============================================================================
 
-def walk_orm_tree(obj, debug: bool = True,
-                  seen: Set = None) -> Generator[object, None, None]:
+def walk_orm_tree(obj,
+                  debug: bool = False,
+                  seen: Set = None,
+                  skip_relationships_always: List[str] = None,
+                  skip_relationships_by_tablename: Dict[str, List[str]] = None,
+                  skip_all_relationships_for_tablenames: List[str] = None,
+                  skip_all_objects_for_tablenames: List[str] = None) \
+        -> Generator[object, None, None]:
     """
     Starting with a SQLAlchemy ORM object, this function walks a
     relationship tree, yielding each of the objects once.
+
+    To skip attributes by name, put the attribute name(s) in skip_attrs_always.
+    To skip by table name, pass skip_attrs_by_tablename as e.g.
+        {'sometable': ['attr1_to_skip', 'attr2_to_skip']}
     """
     # http://docs.sqlalchemy.org/en/latest/faq/sessions.html#faq-walk-objects
+    skip_relationships_always = skip_relationships_always or []  # type: List[str]
+    skip_relationships_by_tablename = skip_relationships_by_tablename or {}  # type: Dict[str, List[str]]  # noqa
+    skip_all_relationships_for_tablenames = skip_all_relationships_for_tablenames or []  # type: List[str]  # noqa
+    skip_all_objects_for_tablenames = skip_all_objects_for_tablenames or []  # type: List[str]  # noqa
     stack = [obj]
     if seen is None:
-        log.critical("New set!")
         seen = set()
     while stack:
         obj = stack.pop(0)
         if obj in seen:
             continue
+        tablename = obj.__tablename__
+        if tablename in skip_all_objects_for_tablenames:
+            continue
         seen.add(obj)
         if debug:
-            log.critical("walk: yielding {!r}", obj)
+            log.debug("walk: yielding {!r}", obj)
         yield obj
         insp = inspect(obj)  # type: InstanceState
         for relationship in insp.mapper.relationships:  # type: RelationshipProperty  # noqa
+            attrname = relationship.key
+            # Skip?
+            if attrname in skip_relationships_always:
+                continue
+            if tablename in skip_all_relationships_for_tablenames:
+                continue
+            if (tablename in skip_relationships_by_tablename and
+                    attrname in skip_relationships_by_tablename[tablename]):
+                continue
+            # Process relationship
             if debug:
-                log.debug("walk: checking relationship {}", relationship)
-            related = getattr(obj, relationship.key)
+                log.debug("walk: following relationship {}", relationship)
+            related = getattr(obj, attrname)
+            if debug and related:
+                log.debug("walk: queueing {!r}", related)
             if relationship.uselist:
                 stack.extend(related)
             elif related is not None:
@@ -422,6 +450,29 @@ def colname_to_attrname_dict(cls) -> Dict[str, str]:
     return reversedict(attrname_to_colname_dict(cls))
 
 
+# =============================================================================
+# Get relationships from an ORM instance
+# =============================================================================
+
+def gen_relationships(obj) -> Generator[Tuple[str, RelationshipProperty, Type],
+                                        None, None]:
+    """
+    Yields tuples of
+        (attrname, RelationshipProperty, related_class)
+    for all relationships of an ORM object.
+    """
+    insp = inspect(obj)  # type: InstanceState
+    # insp.mapper.relationships is of type
+    # sqlalchemy.utils._collections.ImmutableProperties, which is basically
+    # a sort of AttrDict.
+    for attrname, rel_prop in insp.mapper.relationships.items():  # type: Tuple[str, RelationshipProperty]  # noqa
+        related_class = rel_prop.mapper.class_
+        # log.critical("gen_relationships: attrname={!r}, "
+        #              "rel_prop={!r}, related_class={!r}, rel_prop.info={!r}",
+        #              attrname, rel_prop, related_class, rel_prop.info)
+        yield attrname, rel_prop, related_class
+
+
 # =============================================================================
 # Inspect ORM objects (SQLAlchemy ORM)
 # =============================================================================
